"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var react_codemirror2_1 = require("react-codemirror2");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var common_1 = require("../../common");
if (common_1.is.browser) {
    require('codemirror/mode/spreadsheet/spreadsheet.js');
}
var FormulaInput = (function (_super) {
    tslib_1.__extends(FormulaInput, _super);
    function FormulaInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isLoaded: false };
        _this.unmounted$ = new rxjs_1.Subject();
        _this.state$ = new rxjs_1.Subject();
        _this.editorRef = function (ref) { return (_this.editor = ref && ref.editor); };
        _this._events$ = new rxjs_1.Subject();
        _this.events$ = _this._events$.pipe(operators_1.takeUntil(_this.unmounted$), operators_1.share());
        _this.execCommand$ = new rxjs_1.ReplaySubject();
        _this.modifierKeys = {
            alt: false,
            control: false,
            shift: false,
            meta: false,
        };
        _this.fire = function (e) { return _this._events$.next(e); };
        _this.focusHandler = function (isFocused) {
            return function () {
                if (isFocused) {
                    _this.fire({ type: 'INPUT/formula/focus', payload: {} });
                }
                if (!isFocused) {
                    _this.fire({ type: 'INPUT/formula/blur', payload: {} });
                }
            };
        };
        _this.handleBeforeChange = function (editor, data, value) {
            var _a = _this.props, _b = _a.multiline, isMultiLine = _b === void 0 ? false : _b, maxLength = _a.maxLength;
            var change = data;
            var char = change.text[0];
            var modifierKeys = tslib_1.__assign({}, _this.modifierKeys);
            if (char.includes('\t')) {
                var payload_1 = {
                    modifierKeys: modifierKeys,
                    isCancelled: false,
                    cancel: function () {
                        payload_1.isCancelled = true;
                    },
                };
                _this.fire({ type: 'INPUT/formula/tab', payload: payload_1 });
                if (payload_1.isCancelled) {
                    return;
                }
            }
            if (isNewLine(change)) {
                var payload_2 = {
                    modifierKeys: modifierKeys,
                    isCancelled: Boolean(singleLineOnly(change)),
                    cancel: function () {
                        payload_2.isCancelled = true;
                    },
                };
                _this.fire({ type: 'INPUT/formula/enter', payload: payload_2 });
                if (payload_2.isCancelled) {
                    return;
                }
            }
            if (!isMultiLine) {
                var isCancelled = Boolean(singleLineOnly(change));
                if (isCancelled) {
                    return;
                }
            }
            var from = _this.props.value || '';
            var to = value;
            var isMax = maxLength === undefined ? null : to.length === maxLength;
            var isChangeCancelled = false;
            var changedPayload = { from: from, to: to, isMax: isMax, char: char, modifierKeys: modifierKeys };
            var changingPayload = tslib_1.__assign(tslib_1.__assign({}, changedPayload), { get isCancelled() {
                    return isChangeCancelled;
                },
                cancel: function () {
                    isChangeCancelled = true;
                } });
            _this.fire({ type: 'INPUT/formula/changing', payload: changingPayload });
            if (isChangeCancelled) {
                return;
            }
            if (maxLength !== undefined) {
                var clippedValue = formatValue(value, { maxLength: maxLength, isMultiLine: isMultiLine });
                if (clippedValue !== value) {
                    if (change.update && change.origin === '+input') {
                        change.update(undefined, undefined, [clippedValue]);
                        value = clippedValue;
                    }
                }
            }
            _this.fire({ type: 'INPUT/formula/changed', payload: changedPayload });
        };
        return _this;
    }
    FormulaInput.prototype.componentWillMount = function () {
        var _this = this;
        this.state$.pipe(operators_1.takeUntil(this.unmounted$)).subscribe(function (e) { return _this.setState(e); });
        if (this.props.events$) {
            this.events$.subscribe(this.props.events$);
        }
        this.events$
            .pipe(operators_1.filter(function (e) { return e.type === 'INPUT/formula/tab'; }), operators_1.filter(function (e) { return !common_1.value.defaultValue(_this.props.allowTab, true); }), operators_1.map(function (e) { return e.payload; }))
            .subscribe(function (e) { return e.cancel(); });
        var keypress$ = common_1.events.keyPress$.pipe(operators_1.takeUntil(this.unmounted$));
        var modifier$ = keypress$.pipe(operators_1.filter(function (e) { return e.isModifier; }));
        modifier$
            .pipe(operators_1.filter(function (e) { return e.isPressed; }), operators_1.map(function (e) { return e.key.toLowerCase(); }))
            .subscribe(function (key) { return (_this.modifierKeys[key] = true); });
        modifier$
            .pipe(operators_1.filter(function (e) { return !e.isPressed; }), operators_1.map(function (e) { return e.key.toLowerCase(); }))
            .subscribe(function (key) { return (_this.modifierKeys[key] = false); });
    };
    FormulaInput.prototype.componentDidMount = function () {
        var _this = this;
        this.setState({ isLoaded: true }, function () { return _this.init(); });
        this.execCommand$
            .pipe(operators_1.takeUntil(this.unmounted$), operators_1.filter(function () { return Boolean(_this.editor); }))
            .subscribe(function (command) {
            switch (command) {
                case 'focus':
                    _this.editor.focus();
                    break;
                default:
                    _this.editor.execCommand(command);
                    break;
            }
        });
    };
    FormulaInput.prototype.init = function () {
        var _a = this.props, focusOnLoad = _a.focusOnLoad, selectOnLoad = _a.selectOnLoad;
        if (focusOnLoad) {
            this.focus();
        }
        if (selectOnLoad) {
            this.selectAll();
        }
    };
    FormulaInput.prototype.componentWillUnmount = function () {
        this.unmounted$.next();
        this.unmounted$.complete();
    };
    Object.defineProperty(FormulaInput.prototype, "isFocused", {
        get: function () {
            return this.editor ? this.editor.hasFocus() : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormulaInput.prototype, "height", {
        get: function () {
            var _a = this.props, _b = _a.multiline, isMultiLine = _b === void 0 ? false : _b, height = _a.height;
            if (height !== undefined) {
                return height;
            }
            if (!isMultiLine && height === undefined) {
                return 21;
            }
            return;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormulaInput.prototype, "className", {
        get: function () {
            return "platform-formulaInput-" + this.fontSize;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FormulaInput.prototype, "fontSize", {
        get: function () {
            var _a = this.props.fontSize, fontSize = _a === void 0 ? 12 : _a;
            return fontSize;
        },
        enumerable: true,
        configurable: true
    });
    FormulaInput.prototype.redraw = function () {
        this.forceUpdate();
        if (this.editor) {
            this.editor.refresh();
        }
        return this;
    };
    FormulaInput.prototype.focus = function () {
        this.execCommand$.next('focus');
        return this;
    };
    FormulaInput.prototype.selectAll = function () {
        this.execCommand$.next('selectAll');
        return this;
    };
    FormulaInput.prototype.cursorToStart = function () {
        this.execCommand$.next('goDocStart');
        return this;
    };
    FormulaInput.prototype.cursorToEnd = function () {
        this.execCommand$.next('goDocEnd');
        return this;
    };
    FormulaInput.prototype.setGlobalStyles = function () {
        var style = {
            '.CodeMirror': {
                fontSize: this.fontSize,
                background: 'none',
            },
            '.CodeMirror pre': {
                fontFamily: common_1.constants.MONOSPACE.FAMILY,
                fontWeight: 'bold',
            },
        };
        common_1.css.global(style, { prefix: "." + this.className });
    };
    FormulaInput.prototype.render = function () {
        this.setGlobalStyles();
        var _a = this.props, _b = _a.multiline, isMultiLine = _b === void 0 ? false : _b, maxLength = _a.maxLength, _c = _a.mode, mode = _c === void 0 ? 'spreadsheet' : _c;
        var height = this.height;
        var value = formatValue(this.props.value, { maxLength: maxLength, isMultiLine: isMultiLine });
        var styles = {
            base: common_1.css({
                height: height,
                Scroll: isMultiLine,
                overflow: 'hidden',
                lineHeight: 1,
                paddingLeft: 0,
            }),
        };
        var el = this.state.isLoaded && (React.createElement(react_codemirror2_1.Controlled, { ref: this.editorRef, options: { mode: mode }, value: value, onBeforeChange: this.handleBeforeChange, onFocus: this.focusHandler(true), onBlur: this.focusHandler(false) }));
        return (React.createElement("div", tslib_1.__assign({ className: this.className }, common_1.css(styles.base, this.props.style)), el));
    };
    return FormulaInput;
}(React.PureComponent));
exports.FormulaInput = FormulaInput;
function singleLineOnly(change) {
    if (isNewLine(change)) {
        return change.cancel();
    }
    var pastedNewLine = change.origin === 'paste' && typeof change.text === 'object' && change.text.length > 1;
    if (pastedNewLine && change.update) {
        var newText = change.text.join(' ');
        return change.update(undefined, undefined, [newText]);
    }
    return null;
}
function isNewLine(change) {
    return (change.origin === '+input' && typeof change.text === 'object' && change.text.join('') === '');
}
function formatValue(value, options) {
    var maxLength = options.maxLength, _a = options.isMultiLine, isMultiLine = _a === void 0 ? false : _a;
    value = value || '';
    if (!isMultiLine) {
        value = value.replace(/\n/g, '');
    }
    if (maxLength !== undefined && value.length > maxLength) {
        value = value.substr(0, maxLength);
    }
    return value;
}
